@@ -10,7 +10,6 @@ import Image
 import matplotlib.pyplot as plt
 
 from cmpGen import cmpGen
-from InstantRead import read
 from openCSV import openCSV
 import FilenameSort as FS
 import ExcelHelper
@@ -20,7 +19,7 @@ from dplot import dplot
 import GifTiffLoader as GTL
 import SeedWaterSegmenter as SWS
 
-from np_utils import flatten
+from np_utils import flatten,limitInteriorPoints,totuple,removeDuplicates
 
 from MyPath import cprint
 #from pylab import *
@@ -286,6 +285,93 @@ def GetContourValuesLengthsAndSubContoursByFrame(watershed,allValsByFrame):
         cVLSByFrame.append(cVLS)
     return cVLSByFrame
 
+def GetContourValuesLengthsAndSubContoursAndOrderOfSubContoursByFrame(watershed,allValsByFrame):
+    cVLSByFrame = []
+    orderOfSCsByValueByFrame = [] # List of dicts
+    for frame in range(len(watershed)):
+        identifier=0 # unique id for each cVLS
+        cVLS = [] # for each subcountour: [value, length, subcontour points]
+        orderOfSCsByValue = {} # For each cellID, an ordered list of indexes to the subcontours (into cVLS) that reconstruct the contour (negative values mean the sc needs to be flipped)
+        for v in allValsByFrame[frame]:
+            boundingRect=ImageContour.GetBoundingRect(watershed[frame],v)
+            perimeterVals,perimeterList,subContours = ImageContour.GetPerimeterByNeighborVal(watershed[frame],v,boundingRect=boundingRect,getSubContours=True)
+            numSCs=len(perimeterVals)
+            subContoursAdj = [(np.array(sc)+[boundingRect[0][0],boundingRect[1][0]]).tolist() for sc in subContours] # Will need to - 0.5 to line up on an overlay
+            if len(perimeterList)>0:
+                orderOfSCsByValue[v] = []
+                for i in range(numSCs):
+                    newSC = [sorted([v,perimeterVals[i]]),perimeterList[i],subContoursAdj[i],identifier]
+                    matchingCVLS = [ sc for sc in cVLS if sc[0]==newSC[0] ]
+                    matchingCVLS = [ sc for sc in matchingCVLS if sorted([newSC[2][0],newSC[2][-1]]) == sorted([sc[2][0],sc[2][-1]]) ]  # Should only possibly find 1 match...
+                    if matchingCVLS==[]: # New contour!
+                        cVLS.append(newSC)
+                        orderOfSCsByValue[v].append(identifier)
+                        identifier+=1
+                    else:
+                        matchingCVLS[0][1] = min(matchingCVLS[0][1],newSC[1]) # keep the minimum perimeter length...
+                        orderOfSCsByValue[v].append(-matchingCVLS[0][3]) # Minus means it is backwards!        
+        cVLS.sort()
+        IDs = [i[3] for i in cVLS]
+        cVLS = [i[:3] for i in cVLS] # Remove identifiers from cVLS
+        # Reindex after sorting...
+        for v in allValsByFrame[frame]:       #cmp(x,0) -> sign(x) in python
+            orderOfSCsByValue[v] = [cmp(ID,0)*IDs.index(abs(ID)) for ID in orderOfSCsByValue[v]]
+        cVLSByFrame.append(cVLS)
+        orderOfSCsByValueByFrame.append(orderOfSCsByValue)
+    return cVLSByFrame,orderOfSCsByValueByFrame
+
+        ## NOT NEEDED! KEEPING FOR REFERENCE!
+        #for i in range(len(cVLS)-1,0,-1):
+        #    for j in range(i-1,-1,-1): # Loop backwards through the sorted list of cvls's... if the value pair matches, check the endpoints (they will always be reversed for adjacent regions (always go ccw...))
+        #        if cVLS[i][0]!=cVLS[j][0]: # once we no longer match the value pair, we know there are no more matches in the list...
+        #            break
+        #        ######## VERIFY THIS ACTUALLY WORKS THE SAME WAY!!!
+        #        elif (cVLS[i][2][-1],cVLS[i][2][0]]) == (cVLS[j][2][0],cVLS[j][2][-1]):  # if 2 subcoutours are the same,
+        #            if cVLS[j][1]>cVLS[i][1]:
+        #                cVLS[j],cVLS[i] = cVLS[i],cVLS[j] #swap!
+        #            shortest = min(cVLS[j][1],cVLS[i][1])                              # keep only the one with the minimum length computation
+        #            
+        #            cVLS[j][1] = shortest
+        #            del(cVLS[i])
+        #            break
+
+def GetXYListAndPolyListFromCVLS(cVLS,allValsByFrame,orderOfSCsByValueByFrame):
+    numFrames = len(cVLS)
+    xyList = [ sorted(list(set(  tuple(pt) for c in cvlsByVal for pt in c[2]  ))) for cvlsByVal in cVLS ]
+    polyList = []
+    
+    for t in range(numFrames):
+        polyList.append({})
+        for v in allValsByFrame[t]:
+            subContours = [ ( cVLS[t][-index][2][::-1] if index<0 else cVLS[t][index][2] )
+                           for index in orderOfSCsByValueByFrame[t][v] ] # reconstruct the sc's, flipping if index is negative
+            polyList[-1][v] = [ xyList[t].index(totuple(pt)) for sc in subContours for pt in sc ]
+            polyList[-1][v] = removeDuplicates(polyList[-1][v])+[polyList[-1][v][0]] # Remove interior duplication...
+    return xyList,polyList
+
+def GetCVLSWithLimitedPointsBetweenNodes(cVLS,allValsByFrame,splitLength=1,fixedNumInteriorPoints=None):
+    allValues = list(set(tuple(flatten(allValsByFrame))))
+    allPairs = sorted(list(set([tuple(c[0]) for cVLSByFrame in cVLS for c in cVLSByFrame])))
+
+    if fixedNumInteriorPoints:
+        numInteriorPoints = {p:fixedNumInteriorPoints for p in allPairs}
+    else:
+        minLength = {}
+        for p in allPairs:
+            minLength[p] = min([ len(c[2]) for cVLSByFrame in cVLS for c in cVLSByFrame if tuple(c[0])==p ])
+        numInteriorPoints = {}
+        for p in allPairs:
+            numInteriorPoints[p] = minLength[p]//splitLength
+    cVLS2 = deepcopy(cVLS)
+    for cvlsByValue in cVLS2:
+        for c in cvlsByValue:
+            c[2] = limitInteriorPoints(c[2],numInteriorPoints[tuple(c[0])])
+    return cVLS2
+
+def GetXYListAndPolyListWithLimitedPointsBetweenNodes(cVLS,allValsByFrame,orderOfSCsByValueByFrame,splitLength=1,fixedNumInteriorPoints=None):
+    cVLS2 = GetCVLSWithLimitedPointsBetweenNodes(cVLS,allValsByFrame,splitLength,fixedNumInteriorPoints)
+    return GetXYListAndPolyListFromCVLS(cVLS2,allValsByFrame,orderOfSCsByValueByFrame)
+
 def ContourPlotFromImage(im,neighborPairs):
     _=imshow_vr(im,interpolation='nearest',cmap=plt.cm.gray)
     for i,nPair in enumerate(neighborPairs):
@@ -381,13 +467,13 @@ def GetNormalizedArea(areaFromCSV):
 
 def GetWoundValues(d):
     if 'ManualInputs.py' in os.listdir(d):
-        exec(read(d+'/ManualInputs.py').replace('\r','').split('\n')[0])
+        exec( open(d+'/ManualInputs.py','r').read().replace('\r','').split('\n')[0] )
         return woundVals
     else:
         print 'ManualInputs.py does not exist!'
 def GetNeighborValues(d,wv,ind):
     if 'Neighbors.py' in os.listdir(d):
-        exec(read(d+'/Neighbors.py'))
+        exec( open(d+'/Neighbors.py','r').read() )
         allVals = []
         for v in wv:
             n=neighbors[ind][v-2]
@@ -425,28 +511,66 @@ def OutlineMasksPlot(arr,waterArr,cellID,t=0):
         plt.subplot(2,3,i+2)
         plt.imshow(bright,cmap=plt.cm.gray)
 
-def GetTotalIntegratedBrightness(arr,waterArr,cellID,iterations=0):
-    '''Sum the brightness of the original image under the final segmented region'''
+def DilateMinusErode(binaryArr,structure=None,iterations=1):
+    if iterations<1:
+        print 'You must specify at least one iteration!!!'
+        return
+    else:
+        return ndimage.binary_dilation(binaryArr,structure=structure,iterations=iterations) -  \
+                ndimage.binary_erosion(binaryArr,structure=structure,iterations=iterations)
+
+def GetCellVolume(waterArr,cellID,structure=None,iterations=0):
+    sums = []
+    for t in range(len(waterArr)):
+        w = (waterArr[t]==cellID)
+        if iterations>0: #if iterations is >0, w turns into the outline region instead
+            w = DilateMinusErode(w,structure=structure,iterations=iterations)
+        sums.append( w.sum() )
+    return np.array(sums)
+
+def GetTotalIntegratedBrightness(arr,waterArr,cellID,structure=None,iterations=0):
+    '''Sum the brightness of the original image under the final segmented region (2D+time or 3D+time)'''
     sums = []
     bright = np.zeros_like(arr[0])
     for t in range(len(arr)):
         bright[:]=0
         w = (waterArr[t]==cellID)
         if iterations>0: #if iterations is >0, w turns into the outline region instead
-            w = ndimage.binary_dilation(w,iterations=iterations) -  \
-                ndimage.binary_erosion(w,iterations=iterations)
+            w = DilateMinusErode(w,structure=structure,iterations=iterations)
         wh = np.where(w)
         bright[wh]=arr[t][wh]
         sums.append( bright.sum() )
     return np.array(sums)
 
-def GetEdgeIntegratedBrightness(arr,waterArr,cellID,thickness=2):
-    '''Sum the brightness of the original image under the edges of the final segmented region, with a specificed thickness (even integers only)'''
+def GetEdgeIntegratedBrightness(arr,waterArr,cellID,thickness=2,skip3DTopAndBottom=False):
+    '''Sum the brightness of the original image under the edges of the final segmented region, with a specificed thickness (even integers only) (2D+time or 3D+time)'''
     if thickness%2==1 or thickness<2:
         print 'thickness parameter must be an even integer > 0!'
         return
     iterations = thickness//2
-    return GetTotalIntegratedBrightness(arr,waterArr,cellID,iterations=iterations)
+    if not skip3DTopAndBottom:
+        return GetTotalIntegratedBrightness(arr,waterArr,cellID,iterations=iterations)
+    elif arr.ndim==4:
+        structure = np.array([ [[0,1,0],[1,1,1],[0,1,0]] ],np.bool)
+        return GetTotalIntegratedBrightness(arr,waterArr,cellID,structure=structure,iterations=iterations)
+    else:
+        'You can only use skip3DTopAndBottom=True on a 4D array!'
+        return
+
+def GetVolumesAndIntegratedBrightnesses(testArrays,waterArr,cellID,thickness,skip3DBottomAndTop=True):
+    volume = GetCellVolume(waterArr,cellID)
+    volumeEdge = GetCellVolume(waterArr,cellID,structure=[[[0,1,0],[1,1,1],[0,1,0]]],iterations=thickness//2)
+    
+    intensities = [ GetTotalIntegratedBrightness(i,waterArr,cellID=cellID)
+                   for i in testArrays ]
+    edgeIntensities = [ GetEdgeIntegratedBrightness(i,waterArr,cellID,thickness=thickness,skip3DTopAndBottom=skip3DBottomAndTop)
+                       for i in testArrays ]
+    
+    intensitiesPerPixel = [ i/volume for i in intensities ]
+    edgeIntensitiesPerPixel = [ i/volumeEdge for i in edgeIntensities ]
+    
+    return [volume,volumeEdge],intensities,edgeIntensities,intensitiesPerPixel,edgeIntensitiesPerPixel
+
 #####################################################################
 
 
